@@ -13,6 +13,11 @@ Master key renewal process:
 
 For more details, see also :class:`TwoPhaseRenewal`.
 
+.. versionchanged:: 0.2.0
+   ``CloudMasterKeyStore`` is moved from this module to
+   :mod:`geofront.backends.cloud`.
+   See :class:`~.backends.cloud.CloudMasterKeyStore`.
+
 """
 import collections.abc
 import datetime
@@ -21,8 +26,6 @@ import logging
 import os.path
 import threading
 
-from libcloud.storage.base import Container, StorageDriver
-from libcloud.storage.types import ObjectDoesNotExistError
 from paramiko.pkey import PKey
 from paramiko.rsakey import RSAKey
 from paramiko.sftp_client import SFTPClient
@@ -33,8 +36,7 @@ from .keystore import get_key_fingerprint
 from .remote import AuthorizedKeyList, Remote
 from .util import typed
 
-__all__ = ('CloudMasterKeyStore', 'EmptyStoreError',
-           'FileSystemMasterKeyStore', 'MasterKeyStore',
+__all__ = ('EmptyStoreError', 'FileSystemMasterKeyStore', 'MasterKeyStore',
            'PeriodicalRenewal', 'TwoPhaseRenewal',
            'read_private_key_file', 'renew_master_key')
 
@@ -165,8 +167,10 @@ class TwoPhaseRenewal:
         self.sftp_clients = None
 
 
+@typed
 def renew_master_key(servers: collections.abc.Set,
-                     key_store: MasterKeyStore) -> PKey:
+                     key_store: MasterKeyStore,
+                     bits: int=2048) -> PKey:
     """Renew the master key.  It creates a new master key, makes ``servers``
     to authorize the new key, replaces the existing master key with the
     new key in the ``key_store``, and then makes ``servers`` to deauthorize
@@ -179,15 +183,22 @@ def renew_master_key(servers: collections.abc.Set,
     :type servers: :class:`collections.abc.Set`
     :param key_store: the master key store to update
     :type key_store: :class:`MasterKeyStore`
+    :param bits: the number of bits the generated key should be.
+                 it has to be 1024 at least, and a multiple of 256.
+                 2048 by default
+    :type bits: :class:`int`
     :returns: the created new master key
     :rtype: :class:`paramiko.pkey.PKey`
 
+    .. versionadded:: 0.2.0
+       The ``bits`` optional parameter.
+
     """
     logger = logging.getLogger(__name__ + '.renew_master_key')
     logger.info('renew the master key...')
     old_key = key_store.load()
     logger.info('the existing master key: %s', get_key_fingerprint(old_key))
-    new_key = RSAKey.generate(1024)
+    new_key = RSAKey.generate(bits)
     logger.info('created new master key: %s', get_key_fingerprint(new_key))
     logger.info('authorize the new master key...')
     with TwoPhaseRenewal(servers, old_key, new_key):
@@ -211,10 +222,17 @@ class PeriodicalRenewal(threading.Thread):
     :type key_store: :class:`MasterKeyStore`
     :param interval: the interval to renew
     :type interval: :class:`datetime.timedelta`
+    :param bits: the number of bits the generated key should be.
+                 it has to be 1024 at least, and a multiple of 256.
+                 2048 by default
+    :type bits: :class:`int`
     :param start: whether to start the background thread immediately.
                   :const:`True` by default
     :type start: :class:`bool`
 
+    .. versionadded:: 0.2.0
+       The ``bits`` optional parameter.
+
     """
 
     @typed
@@ -222,11 +240,13 @@ class PeriodicalRenewal(threading.Thread):
                  servers: collections.abc.Set,
                  key_store: MasterKeyStore,
                  interval: datetime.timedelta,
+                 bits: int=2048,
                  start: bool=True):
         super().__init__()
         self.servers = servers
         self.key_store = key_store
         self.interval = interval
+        self.bits = bits
         self.terminated = threading.Event()
         if self.start:
             self.start()
@@ -238,7 +258,7 @@ class PeriodicalRenewal(threading.Thread):
             terminated.wait(seconds)
             if terminated.is_set():
                 break
-            renew_master_key(self.servers, self.key_store)
+            renew_master_key(self.servers, self.key_store, self.bits)
 
     def terminate(self):
         """Graceful termination."""
@@ -282,95 +302,3 @@ class FileSystemMasterKeyStore(MasterKeyStore):
     @typed
     def save(self, master_key: PKey):
         master_key.write_private_key_file(self.path)
-
-
-class CloudMasterKeyStore(MasterKeyStore):
-    """Store the master key into the cloud object storage e.g. AWS S3_.
-    It supports more than 20 cloud providers through the efforts of Libcloud_.
-    ::
-
-        from geofront.masterkey import CloudMasterKeyStore
-        from libcloud.storage.types import Provider
-        from libcloud.storage.providers import get_driver
-
-        driver_cls = get_driver(Provider.S3)
-        driver = driver_cls('api key', 'api secret key')
-        container = driver.get_container(container_name='my-master-key-bucket')
-        MASTER_KEY_STORE = CloudMasterKeyStore(container)
-
-    :param driver: the libcloud storage driver
-    :type driver: :class:`libcloud.storage.base.StorageDriver`
-    :param container: the block storage container
-    :type container: :class:`libcloud.storage.base.Container`
-    :param object_name: the object name to use
-    :type object_name: :class:`str`
-
-    .. seealso::
-
-       `Object Storage`__ --- Libcloud
-          Storage API allows you to manage cloud object storage and
-          services such as Amazon S3, Rackspace CloudFiles,
-          Google Storage and others.
-
-    .. _S3: http://aws.amazon.com/s3/
-    .. _Libcloud: http://libcloud.apache.org/
-    __ https://libcloud.readthedocs.org/en/latest/storage/
-
-    """
-
-    @typed
-    def __init__(self,
-                 driver: StorageDriver,
-                 container: Container,
-                 object_name: str):
-        self.driver = driver
-        self.container = container
-        self.object_name = object_name
-
-    @typed
-    def load(self) -> PKey:
-        try:
-            obj = self.driver.get_object(self.container.name, self.object_name)
-        except ObjectDoesNotExistError:
-            raise EmptyStoreError()
-        with io.BytesIO() as buffer_:
-            for chunk in self.driver.download_object_as_stream(obj):
-                if isinstance(chunk, str):  # DummyDriver yields str, not bytes
-                    chunk = chunk.encode()
-                buffer_.write(chunk)
-            buffer_.seek(0)
-            with io.TextIOWrapper(buffer_) as tio:
-                return read_private_key_file(tio)
-
-    @typed
-    def save(self, master_key: PKey):
-        with io.StringIO() as buffer_:
-            master_key.write_private_key(buffer_)
-            pem = buffer_.getvalue()
-        self.driver.upload_object_via_stream(
-            self._countable_iterator([pem]),
-            self.container,
-            self.object_name,
-            {'content_type': 'application/x-pem-key'}
-        )
-
-    class _countable_iterator:
-        """libcloud's storage driver takes an iterator as stream,
-        but some drivers e.g. dummy driver try calling :func:`len()`
-        to the iterator.  This adapter workarounds the situation.
-
-        """
-
-        @typed
-        def __init__(self, sequence: collections.abc.Sequence):
-            self.iterator = iter(sequence)
-            self.length = len(sequence)
-
-        def __len__(self):
-            return self.length
-
-        def __iter__(self):
-            return self
-
-        def __next__(self):
-            return next(self.iterator)
\ No newline at end of file
