@@ -51,7 +51,6 @@ import warnings
 from flask import (Flask, Response, current_app, json, jsonify, make_response,
                    request, url_for)
 from paramiko.pkey import PKey
-from paramiko.rsakey import RSAKey
 from paramiko.ssh_exception import SSHException
 from waitress import serve
 from waitress.adjustments import Adjustments
@@ -65,9 +64,10 @@ from .identity import Identity
 from .keystore import (DuplicatePublicKeyError, KeyStore, KeyTypeError,
                        format_openssh_pubkey, get_key_fingerprint,
                        parse_openssh_pubkey)
-from .masterkey import (EmptyStoreError, MasterKeyStore, PeriodicalRenewal,
-                        renew_master_key)
-from .remote import Remote, authorize
+from .masterkey import MasterKeyStore, PeriodicalRenewal
+from .regen import RegenError, main_parser as regen_main_parser, regenerate
+from .remote import (DefaultPermissionPolicy, PermissionPolicy, Remote,
+                     authorize)
 from .team import AuthenticationError, Team
 from .util import typed
 from .version import VERSION
@@ -76,10 +76,11 @@ __all__ = ('AUTHORIZATION_TIMEOUT',
            'FingerprintConverter', 'Token', 'TokenIdConverter',
            'add_public_key', 'app', 'authenticate', 'authorize_remote',
            'create_access_token', 'delete_public_key', 'get_identity',
-           'get_key_store', 'get_master_key_store', 'get_public_key',
-           'get_remote_set', 'get_team', 'get_token_store',
+           'get_key_store', 'get_master_key_store', 'get_permission_policy',
+           'get_public_key', 'get_remote_set', 'get_team', 'get_token_store',
            'list_public_keys', 'main', 'main_parser', 'master_key',
-           'public_key', 'remote_dict', 'server_version', 'token')
+           'public_key', 'remote_dict', 'server_endpoint', 'server_version',
+           'token')
 
 
 #: (:class:`datetime.timedelta`) How long does each temporary authorization
@@ -137,6 +138,8 @@ app.url_map.converters.update(
     fingerprint=FingerprintConverter
 )
 app.config.update(  # Config defaults
+    PERMISSION_POLICY=DefaultPermissionPolicy(),
+    MASTER_KEY_BITS=2048,
     MASTER_KEY_RENEWAL=datetime.timedelta(days=1),
     TOKEN_EXPIRE=datetime.timedelta(days=7)
 )
@@ -154,6 +157,42 @@ def server_version(response: Response) -> Response:
     return response
 
 
+@app.route('/')
+def server_endpoint():
+    """The endpoint of HTTP API which provide the url to create a new token.
+
+    .. code-block:: http
+
+       GET / HTTPS/1.1
+       Accept: application/json
+
+    .. code-block:: http
+
+       HTTP/1.0 200 OK
+       Content-Type: application/json
+       Link: <https://example.com/tokens/>; rel=tokens
+
+       {
+         "tokens_url": "https://example.com/tokens/"
+       }
+
+    :resheader Link: the url to create a new token.  the equivalent to
+                     the response content
+    :status 200: when the server is available
+
+    .. versionadded:: 0.2.0
+
+    """
+    tokens_url = url_for(
+        'token',
+        token_id='TOKEN_ID',
+        _external=True
+    ).replace('/TOKEN_ID/', '/')  # FIXME
+    response = jsonify(tokens_url=tokens_url)
+    response.headers.add('Link', '<{}>'.format(tokens_url), rel='tokens')
+    return response
+
+
 def get_team() -> Team:
     """Get the configured team implementation, an instance of
     :class:`.team.Team`.
@@ -262,7 +301,7 @@ def create_access_token(token_id: str):
     response = jsonify(next_url=next_url)
     assert isinstance(response, Response)
     response.status_code = 202
-    response.headers['Link'] = '<{0}>; rel=next'.format(next_url)
+    response.headers.add('Link', '<{}>'.format(next_url), rel='next')
     response.expires = (datetime.datetime.now(datetime.timezone.utc) +
                         datetime.timedelta(seconds=timeout))
     return response
@@ -319,7 +358,7 @@ def authenticate(token_id: str):
         ),
         html.body(
             html.h1(html.dfn('Geofront:'), ' Authentication success'),
-            html.p('Close the browser tab, and back to CLI.')
+            html.p('You may close the browser, and go back to the CLI.')
         )
     )
 
@@ -395,22 +434,44 @@ def token(token_id: str):
 
        HTTPS/1.0 200 OK
        Content-Type: application/json
+       Link: <https://example.com/tokens/0123456789abcdef/remo...>; rel=remotes
+       Link: <https://example.com/tokens/0123456789abcdef/keys/>; rel=keys
+       Link: <https://example.com/tokens/0123456789abcdef/ma...>; rel=masterkey
 
        {
          "identifier": "dahlia",
-         "team_type": "geofront.backends.github.GitHubOrganization"
+         "team_type": "geofront.backends.github.GitHubOrganization",
+         "remotes_url": "https://example.com/tokens/0123456789abcdef/remotes/",
+         "keys_url": "https://example.com/tokens/0123456789abcdef/keys/",
+         "master_key_url": "https://example.com/tokens/0123456789abcdef/mas..."
        }
 
     :param token_id: the token id that holds the identity
     :type token_id: :class:`str`
+    :resheader Link: the url to list remotes (``rel=remotes``), public keys
+                     (``rel=keys``), and master key (``rel=masterkey``)
     :status 200: when the token is authenticated
 
+    .. versionchanged:: 0.2.0
+       The response contains ``"remotes_url"``, ``"keys_url"``, and
+       ``"master_key_url"``, and equivalent three :mailheader:`Link` headers.
+
     """
     identity = get_identity(token_id)
-    return jsonify(
+    links = {
+        'remotes': url_for('list_remotes', token_id=token_id, _external=True),
+        'keys': url_for('list_public_keys', token_id=token_id, _external=True),
+        'master_key': url_for('master_key', token_id=token_id, _external=True)
+    }
+    response = jsonify(
+        {rel + '_url': url for rel, url in links.items()},
         team_type='{0.__module__}.{0.__qualname__}'.format(identity.team_type),
-        identifier=identity.identifier
+        identifier=identity.identifier,
     )
+    for rel, href in links.items():
+        response.headers.add('Link', '<{}>'.format(href),
+                             rel=rel.replace('_', ''))
+    return response
 
 
 def get_master_key_store() -> MasterKeyStore:
@@ -724,6 +785,30 @@ def get_remote_set() -> collections.abc.Mapping:
     )
 
 
+def get_permission_policy() -> PermissionPolicy:
+    """Get the configured permission policy.
+
+    :return: the configured permission policy
+    :rtype: :class:`~.remote.PermissionPolicy`
+    :raise RuntimeError: if ``'PERMISSION_POLICY'`` is not configured,
+                         or it's not an instance of
+                         :class:`~.remote.PermissionPolicy`
+
+    .. versionadded:: 0.2.0
+
+    """
+    try:
+        policy = app.config['PERMISSION_POLICY']
+    except KeyError:
+        raise RuntimeError('PERMISSION_POLICY configuration is not present')
+    if isinstance(policy, PermissionPolicy):
+        return policy
+    raise RuntimeError(
+        'PERMISSION_POLICY configuration must be an instance of {0.__module__}'
+        '.{0.__qualname__}, not {1!r}'.format(PermissionPolicy, policy)
+    )
+
+
 def remote_dict(remote: Remote) -> collections.abc.Mapping:
     """Convert a ``remote`` to a simple dictionary that can be serialized
     to JSON.
@@ -768,8 +853,11 @@ def list_remotes(token_id: str):
     .. todo:: Filter by query string.
 
     """
-    get_identity(token_id)  # 404/410 if not authenticated
-    remotes = get_remote_set()
+    team = get_team()
+    identity = get_identity(token_id)  # 404/410 if not authenticated
+    groups = team.list_groups(identity)
+    policy = get_permission_policy()
+    remotes = policy.filter(get_remote_set(), identity, groups)
     return jsonify(
         {alias: remote_dict(remote) for alias, remote in remotes.items()}
     )
@@ -805,10 +893,12 @@ def authorize_remote(token_id: str, alias: str):
     :status 404: (``not-found``) when there's no such remote
 
     """
+    team = get_team()
     identity = get_identity(token_id)
     key_store = get_key_store()
     master_key_store = get_master_key_store()
     remotes = get_remote_set()
+    policy = get_permission_policy()
     try:
         remote = remotes[alias]
     except KeyError:
@@ -817,7 +907,15 @@ def authorize_remote(token_id: str, alias: str):
             message='No such remote alias: {}.'.format(alias)
         )
         response.status_code = 404
-        raise HTTPException(response=response)
+        return response
+    if not policy.permit(remote, identity, team.list_groups(identity)):
+        response = jsonify(
+            error='forbidden',
+            message='Remote {0} is disallowed.  Contact the system '
+                    'administrator.'.format(alias)
+        )
+        response.status_code = 403
+        return response
     public_keys = key_store.list_keys(identity)
     master_key = master_key_store.load()
     remote_mapping = remote_dict(remote)
@@ -841,27 +939,25 @@ def authorize_remote(token_id: str, alias: str):
 
 def main_parser() -> argparse.ArgumentParser:  # pragma: no cover
     """Create an :class:`~argparse.ArgumentParser` object for
-    :program:`geofront-server` CLI program.
+    :program:`geofront-server` CLI program.  It also is used for
+    documentation through `sphinxcontrib-autoprogram`__.
 
     :return: a properly configured :class:`~argparse.ArgumentParser`
     :rtype: :class:`argparse.ArgumentParser`
 
+    __ https://pythonhosted.org/sphinxcontrib-autoprogram/
+
     """
     parser = argparse.ArgumentParser(
         description='Simple SSH key management service'
     )
-    parser.add_argument('config',
-                        metavar='FILE',
-                        help='geofront configuration file (Python script)')
+    parser = regen_main_parser(parser)
     parser.add_argument('-H', '--host',
                         default='0.0.0.0',
                         help='host to bind [%(default)s]')
     parser.add_argument('-p', '--port',
                         default=5000,
                         help='port to bind [%(default)s]')
-    parser.add_argument('--create-master-key',
-                        action='store_true',
-                        help='create a new master key if no master key yet')
     parser.add_argument('--renew-master-key',
                         action='store_true',
                         help='renew the master key before the server starts. '
@@ -872,13 +968,6 @@ def main_parser() -> argparse.ArgumentParser:  # pragma: no cover
                              'url_scheme via the X-Forwarded-Proto header. '
                              'useful when it runs behind reverse proxy. '
                              '-d/--debug option disables this option')
-    parser.add_argument('-d', '--debug',
-                        action='store_true',
-                        help='debug mode.  note that this option may make '
-                             'slowdown')
-    parser.add_argument('-v', '--version',
-                        action='version',
-                        version='%(prog)s ' + VERSION)
     return parser
 
 
@@ -897,21 +986,27 @@ def main():  # pragma: no cover
     logger.addHandler(handler)
     logger.setLevel(level)
     master_key_store = get_master_key_store()
-    servers = frozenset(get_remote_set().values())
+    remote_set = get_remote_set()
+    servers = frozenset(remote_set.values())
+    master_key_bits = app.config['MASTER_KEY_BITS']
+    if not isinstance(master_key_bits, int):
+        parser.error('MASTER_KEYS_BITS configuration must be an integer, not '
+                     + repr(master_key_bits))
+    elif master_key_bits < 1024:
+        parser.error('MASTER_KEY_BITS has to be 1024 at least.')
+    elif master_key_bits % 256:
+        parser.error('MASTER_KEY_BITS has to be a multiple of 256.')
     try:
-        key = master_key_store.load()
-    except EmptyStoreError:
-        if args.create_master_key or args.renew_master_key:
-            logger.warn('no master key;  create one...')
-            key = RSAKey.generate(1024)
-            master_key_store.save(key)
-            logger.info('created new master key: %s', get_key_fingerprint(key))
-        else:
-            parser.error('no master key;  try --create-master-key option '
-                         'if you want to create one')
-    else:
-        if args.renew_master_key and not os.environ.get('WERKZEUG_RUN_MAIN'):
-            renew_master_key(servers, master_key_store)
+        regenerate(
+            master_key_store,
+            remote_set,
+            master_key_bits,
+            create_if_empty=args.create_master_key or args.renew_master_key,
+            renew_unless_empty=(args.renew_master_key and
+                                not os.environ.get('WERKZEUG_RUN_MAIN'))
+        )
+    except RegenError as e:
+        parser.error(str(e))
     master_key_renewal_interval = app.config['MASTER_KEY_RENEWAL']
     if not (master_key_renewal_interval is None or
             isinstance(master_key_renewal_interval, datetime.timedelta)):
@@ -923,7 +1018,8 @@ def main():  # pragma: no cover
         master_key_renewal = PeriodicalRenewal(
             servers,
             master_key_store,
-            master_key_renewal_interval
+            master_key_renewal_interval,
+            master_key_bits
         )
     waitress_options = {}
     if args.trusted_proxy:
